% In this example we apply the LNWS method to denoise a simulated noisy
% Doppler signal with sample size N=512 and signal-to-noise ratio SNR=5.
%
% 3 options for estimation can be used: 1) Posterior mean 
%                                       2) Posterior median
%                                       3) Bayes factor
%
% The hyperparameters are estimated through empirical Bayes method
% numerically maximizing the marginal distribution
% 
%  The program calls to:
%           marg_lnws.m -- Calculates the value of the marginal 
%                          log-likelihood for input hyperparameter values.
%           bayesrule_lnws.m -- Computes the posterior mean for the LNWS 
%                               method.
%           bayesrule_lnws_median.m -- Computes the posterior median for
%                                      the LNWS method.
%           bayesrule_lnws_bf.m -- Computes the Bayes factor for the LNWS
%                                  method.
%--------------------------------------------------------------------------
clear all;
close all;
set(0, 'DefaultAxesFontSize', 16);
  
N = 512;  % sample sise
SNR = 5;  % signal-to-noise ratio
coarsest = floor(log2( log (N))) + 1;  % coarsest level of decomposition
finest_lev = log2(N)-1;  % finest level

seed = 1;  % set rn generator seed
randn('state', seed );  % initialize random number generator
signal = MakeSignal('Doppler', N);  % create Doppler signal
sigma = std(signal);
signal = signal .* SNR/sigma;  % set SNR ratio
nois = randn(N,1,1);     
data = signal + nois';  % add noise to signal

filt = MakeONFilter('Symmlet', 8);  % wavelet filter

data_wd = FWT_PO (data, coarsest-1, filt);  % wavelet transform performed one level after coarsest to get parents for detail coefficients
data_smowd = data_wd;
niz = data_wd(dyad(finest_lev));
sigma_hat = mad(niz,1)/0.6745;  % estimate noise

% Forming "thresholding statistics"
for i = coarsest:1:finest_lev
    ix=dyad(i);
    for j=ix(1:2:end)
        thetaj(round(j/2))=(data_wd(j)^2+data_wd(j+1)^2+data_wd(round(j/2))^2)/sigma_hat^2;
    end;
end;

% Empirical Bayes estimation of hyperparameters
% minimizing the negative of the marginal log-likelihood by 'interior-point' method
x0 = [repmat(0.5,1,finest_lev-coarsest+1) 0.001];
lb=[repmat(0,1,finest_lev-coarsest+1) 0];
ub=[repmat(1,1,finest_lev-coarsest+1) Inf];
options = optimset('Algorithm','interior-point','Display','off');
par_opt = fmincon(@(par) marg_lnws(par,thetaj,coarsest),x0,[],[],[],[],lb,ub,[],options);

% Bayesian shrinkage of energies
for i = (coarsest-1):1:(finest_lev-1),   
    thetajj=thetaj(dyad(i));
    th_tr = thetajj;
    nn = length(th_tr);   
    for  j = 1:nn,
        th_tr(j) = bayesrule_lnws(thetajj(j), par_opt(i-(coarsest-2)), par_opt(end) ); % using posterior mean
        %th_tr(j) = bayesrule_lnws_median(thetajj(j),par_opt(i-(coarsest-2)), par_opt(end) ); % using posterior median
        %th_tr(j) = bayesrule_lnws_bf(thetajj(j), par_opt(i-(coarsest-2)), par_opt(end) ); % using Bayes factor
    end;
    thetaj_smo(dyad(i))= th_tr;
    shrink(dyad(i))=sqrt(th_tr./thetajj);
end;

% Shrinkage of wavelet coefficients
shrink_factor(1:2:N-1)=shrink;
shrink_factor(2:2:N)=shrink;
shrink_factor(1:2^coarsest)=1; % preserve coarsest level  
data_smowd=shrink_factor.*data_wd;
    
                 
sout = IWT_PO (data_smowd, coarsest-1, filt); % inverse wavelet transfrom

plot(sout); 
hold on;
plot(signal,'r-');
axis tight;
title('Denoised Doppler signal using the LNWS method');




